/*-
 * Copyright (c) 2010 Mikolaj Golub <to.my.trociny@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Inefficient implementation of refcount.
 */

#ifndef _REFCOUNT_H_
#define _REFCOUNT_H_

#ifdef _HAVE_SYS_REFCOUNT_H
#include <sys/refcount.h>
#else /* ! _HAVE_SYS_REFCOUNT_H */

#include <stdbool.h>

#include "synch.h"

static pthread_mutex_t _refcount_lock;

static __inline void
refcount_init(volatile u_int *count, u_int value)
{
	static bool initialised = false;

	if (!initialised) {
		synch_mtx_init(&_refcount_lock);
		initialised = true;
	}

	*count = value;
}

static __inline void
refcount_acquire(volatile u_int *count)
{

	synch_mtx_lock(&_refcount_lock);
	(*count)++;
	synch_mtx_unlock(&_refcount_lock);
}

static __inline int
refcount_release(volatile u_int *count)
{

	synch_mtx_lock(&_refcount_lock);
	(*count)--;
	synch_mtx_unlock(&_refcount_lock);
	return (*count == 0);
}

#endif	/* _HAVE_SYS_REFCOUNT_H */

#endif	/* ! _REFCOUNT_H_ */
